# Copyright 2018 Cumulus Networks, Inc.  All Rights reserved.
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from __future__ import absolute_import
import psutil
import os
from netq_lib.orm.redisdb.models import AgentStats
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.agent_stats_monitor import AgentStatsMon


def process_agent_stats(cmd_output, svc_failed=False, **kwargs):

    agent_stats_obj = AgentStats()
    agent_stats_mon_obj = AgentStatsMon.get_instance()

    agent_stats_obj.agent_util = {'agent_pid': agent_stats_mon_obj.agent_util_stats.get('agent_pid', os.getpid()),
                                  'cpu_util': agent_stats_mon_obj.agent_util_stats.get('cpu_util', 0.0) / agent_stats_mon_obj.agent_util_stats.get('count', 1),
                                  'mem_util': agent_stats_mon_obj.agent_util_stats.get('mem_util', 0.0) / agent_stats_mon_obj.agent_util_stats.get('count', 1)}

    agent_stats_obj.message_count = agent_stats_mon_obj.message_count
    agent_stats_obj.msg_type_count = agent_stats_mon_obj.msg_type_count
    agent_stats_obj.msgtype_fullstate_count = agent_stats_mon_obj.msgtype_fullstate_count
    agent_stats_obj.pq_full_count = agent_stats_mon_obj.pq_full_count
    agent_stats_obj.agent_pause_count = agent_stats_mon_obj.agent_pause_count

    agent_stats_mon_obj.reset_agent_stats_counters()

    agent_stats_obj.save()
    return ServiceStatus.OK


def process_agent_util_stats(cmd_output, svc_failed=False, **kwargs):

    agent_util_mon = AgentStatsMon.get_instance()
    agent_process = psutil.Process(os.getpid())

    agent_util_mon.agent_util_stats['agent_pid'] = os.getpid()
    agent_util_mon.agent_util_stats['cpu_util'] = agent_util_mon.agent_util_stats.get('cpu_util', 0.0) + agent_process.cpu_percent(interval=0.1)
    agent_util_mon.agent_util_stats['count'] = agent_util_mon.agent_util_stats.get('count', 0) + 1

    total_mem = psutil.virtual_memory().total
    agent_util_mon.agent_util_stats['mem_util'] = agent_util_mon.agent_util_stats.get('mem_util', 0.0) + \
                                                   float((agent_process.memory_info().vms * 100)) / total_mem

    return ServiceStatus.OK
